/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */ 

package com.jboss.dvd.seam;

import java.io.Serializable;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.jboss.seam.Seam;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.contexts.Context;
import org.jboss.seam.core.Actor;
import org.jboss.seam.core.FacesMessages;

//jboss-sso integration
import org.jboss.security.saml.SSOUser;
import javax.servlet.http.HttpServletRequest;
import javax.faces.context.*;

@Stateless
@Name("login")
public class LoginAction 
    implements Login,
               Serializable
{
    private static final String USER_VAR = "currentUser";

    @PersistenceContext
    private EntityManager em;

    @In
    Context sessionContext;
    
    @In(create=true) 
    Actor actor;
    
    String username = "";
    String password = "";

    public String getUserName() {
        return username;
    }
    public void setUserName(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }
    public void setPassword(String password) {
        this.password = password;
    }
    
    public String login() {
        try {
            User found =  
                (User) em.createQuery("select u from User u where u.userName = :userName and u.password = :password")
                .setParameter("userName", username)
                .setParameter("password", password)
                .getSingleResult();

            sessionContext.set(USER_VAR, found);
            
            
            //jboss-sso integration
            SSOUser ssoUser = new SSOUser(found.getUserName(),null);
            sessionContext.set("jboss_sso_user",ssoUser);
            
            
            actor.setId(username);
            
            if (found instanceof Admin) {
                actor.getGroupActorIds().add("shippers");
                actor.getGroupActorIds().add("reviewers");
                return "admin";
            } else {
                //jboss-sso integration
                return "home"; // redisplay the current page
            }
        } catch (Exception e) {
            // this message is lost in the session invalidation
            FacesMessages.instance().addFromResourceBundle("loginErrorPrompt");
            Seam.invalidateSession();
            return "home";
        }
    }

    public String logout() {
        Seam.invalidateSession();
        sessionContext.set(USER_VAR, null);
        sessionContext.set("loggedIn", null);
        
        //jboss-sso integration
        sessionContext.set("jboss_sso_user",null);
        return "logout";
    }

    private User currentUser() 
    {
        if(sessionContext.get(USER_VAR)!=null)
        {
            return (User) sessionContext.get(USER_VAR); 
        }
        
        //jboss-sso integration here....
        //check for existence of sso user...and perform 
        //an auto-login if possible
        SSOUser ssoUser = (SSOUser)sessionContext.get("jboss_sso_user");
        if(ssoUser!=null && ssoUser.getUserName()!=null)
        {
            User found =  
                (User) em.createQuery("select u from User u where u.userName = :userName")
                .setParameter("userName",ssoUser.getUserName())
                .getSingleResult();

            sessionContext.set(USER_VAR, found);
                            
            
            actor.setId(ssoUser.getUserName());            
            if (found instanceof Admin) {
                actor.getGroupActorIds().add("shippers");
                actor.getGroupActorIds().add("reviewers");
            } 
        }        
        return (User) sessionContext.get(USER_VAR);
    }

    public boolean isLoggedIn() {
        return this.currentUser()!=null;
    }

    public boolean isCustomer() {
        User user = currentUser();
        return (user!=null) && (user instanceof Customer);
    }

    public boolean isAdmin() {
        User user = currentUser();
        return (user!=null) && (user instanceof Admin);
    }
    
    public String adminCheck() {
        return isAdmin() ? null : "home";
    }
    
    //jboss-sso integration
    public boolean isRefreshActive()
    {
        boolean refreshActive = false;
        
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        String refresh = (String)externalContext.getRequestParameterMap().get("refresh");
        if(refresh!=null)
        {
            refreshActive = true;            
        }
        
        return refreshActive;
    }
    
    //jboss-sso integration
    public String getRequestURL()
    {
        
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        HttpServletRequest request = (HttpServletRequest)externalContext.getRequest();
        
        return request.getRequestURL().toString();
    }
    
    //jboss-sso integration
    public String getReservationPartner()
    {
        //return "http://hotels.jboss.com:8080/seam-booking/main.seam";
        return "http://hotels.redhat.com:8080/seam-booking/main.seam";
    }
        
}
